@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gnome-print-ps2.c: A Postscript driver for GnomePrint
  *
@@ -34,7 +34,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <locale.h>
 
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_misc.h>
@@ -414,6 +413,46 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 
 #define EPSILON 1e-9
 
+static gboolean
+gnome_print_ps2_show (GnomePrintPs2 *ps2, GnomePosGlyphList const *pgl, int start, int len)
+{
+	gboolean problem, needs_x = TRUE, needs_y = FALSE;
+	int i;
+
+	problem  = gnome_print_ps2_puts (ps2, ")" EOL);
+	/* Do we need explicit y positioning */
+	for (i = start + 1; i < start + len; i++)
+		if (fabs (pgl->glyphs[i].y - pgl->glyphs[i-1].y) > EPSILON) {
+			needs_y = TRUE;
+			break;
+		}
+
+	if (needs_x || needs_y) {
+		problem |= gnome_print_ps2_putc (ps2, '[');	/* Build array */
+		for (i = start + 1; i < start + len; i++) {
+			if (needs_x) {
+				problem = gnome_print_ps2_print_double (ps2, "%g",
+					 pgl->glyphs[i].x - pgl->glyphs[i-1].x);
+				if (needs_y)
+					problem |= gnome_print_ps2_putc (ps2, ' ');
+			}
+			if (needs_y)
+				problem |= gnome_print_ps2_print_double (ps2, "%g",
+					 pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			problem |= gnome_print_ps2_putc (ps2, ' ');
+		}
+		if (needs_x) {
+			if (needs_y)
+				problem |= gnome_print_ps2_puts (ps2, "0 0] xyshow" EOL);
+			else
+				problem |= gnome_print_ps2_puts (ps2, "0] xshow" EOL);
+		} else
+			problem |= gnome_print_ps2_puts (ps2, "0] yshow" EOL);
+	} else
+		problem |= gnome_print_ps2_puts (ps2, "show" EOL);
+	return problem;
+}
+
 static gint
 gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphList *gl)
 {
@@ -421,8 +460,9 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 	GnomePrintPs2 *ps2;
 	GnomePosGlyphList *pgl;
 	gboolean identity;
+	gint s, master_glyph, glyph_page;
+	gint glyph; /* Glyph in subfont */
 	gdouble dx, dy;
-	gint s;
 	gboolean problem;
 
 	ps2 = (GnomePrintPs2 *) pc;
@@ -459,10 +499,13 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 	for (s = 0; s < pgl->num_strings; s++) {
 		GnomePosString * ps;
 		GnomeFont *font;
+		int start, len;
 		gint i, current_page = 0; /* placate compiler */
 
 		ps = pgl->strings + s;
 		font = gnome_rfont_get_font (ps->rfont);
+		start = ps->start;
+		len   = ps->length;
 
 		if (!NEEDED_SUBSETTING(font)) {
 			problem = gnome_print_ps2_set_font_real (ps2, font, FALSE, 0);
@@ -483,27 +526,39 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 		problem |= gnome_print_ps2_puts (ps2, " m" EOL);
 
 		g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-		/* Build string */
 
+		/* Build string */
 		if (NEEDED_SUBSETTING(font)) {
 			for (i = ps->start; i < ps->start + ps->length; i++) {
-				gint master_glyph, glyph, page;
+				/* Glyph position in parent font */
                                 master_glyph = pgl->glyphs[i].glyph;
-                                page = master_glyph / 255;
+				/* Pages of 256 glyphs, with leading null */
+                                glyph_page = master_glyph / 255;
 
-                                if (i == ps->start) {
-					current_page = page;
-					problem  = gnome_print_ps2_set_font_real (ps2, font, TRUE, page);
+				if (i == ps->start) {		/* Initially */
+					current_page = glyph_page;
+					problem  = gnome_print_ps2_set_font_real (ps2, font, TRUE, glyph_page);
 					problem |= gnome_print_ps2_putc (ps2, '(');
-				} else if (current_page != page) {
-                                        current_page = page;
-                                        problem = gnome_print_ps2_puts (ps2, ")show" EOL);
-					problem |= gnome_print_ps2_set_font_real (ps2, font, TRUE, page);
+				} else if (current_page != glyph_page) { /* There's a ttf subfont to be set*/
+                                        current_page = glyph_page;
+					problem = gnome_print_ps2_show (ps2, pgl, start, len);
+					len -= i - start;
+					start = i;
+					problem |= gnome_print_ps2_set_font_real (ps2, font, TRUE, glyph_page);
 					problem |= gnome_print_ps2_putc (ps2, '(');
 				}
+
 				gnome_font_face_pso_mark_glyph (ps2->selected_font->pso, master_glyph);
+
+				/* For each sub ttf font the first glyph is  */
+				/* null and 255th one is a valid glyph, also */
+				/* 1 needed to be added to each subglyph     */
+				/* for taking into account first null glyph  */
+				/* added to every subfont	             */
+
 				glyph = master_glyph ? master_glyph % 255 + 1 : master_glyph;
 				problem |= gnome_print_ps2_print_octal_byte (ps2, glyph);
+
 				g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
 			}
 		} else if (ps2->selected_font->pso->encodedbytes == 1) {
@@ -520,42 +575,17 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 			problem = gnome_print_ps2_putc (ps2, '(');
 			/* 16-bit encoding */
 			for (i = ps->start; i < ps->start + ps->length; i++) {
-				gint glyph, page;
 				gnome_font_face_pso_mark_glyph (ps2->selected_font->pso, pgl->glyphs[i].glyph);
 				glyph = pgl->glyphs[i].glyph & 0xff;
-				page = (pgl->glyphs[i].glyph >> 8) & 0xff;
-				problem  = gnome_print_ps2_print_octal_byte (ps2, page);
+				glyph_page = (pgl->glyphs[i].glyph >> 8) & 0xff;
+				problem  = gnome_print_ps2_print_octal_byte (ps2, glyph_page);
 				problem |= gnome_print_ps2_print_octal_byte (ps2, glyph);
 				g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
 			}
 		}
 
-		if (!NEEDED_SUBSETTING(font)) {
-			problem = gnome_print_ps2_puts (ps2, ")" EOL);
-			g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-			/* Build array */
-			problem = gnome_print_ps2_putc (ps2, '[');
-			g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-			for (i = ps->start + 1; i < ps->start + ps->length; i++) {
-				problem = gnome_print_ps2_print_double
-					(ps2, "%g",
-					 pgl->glyphs[i].x - pgl->glyphs[i-1].x);
-				problem |= gnome_print_ps2_putc (ps2, ' ');
-				problem |= gnome_print_ps2_print_double
-					(ps2, "%g",
-					 pgl->glyphs[i].y - pgl->glyphs[i-1].y);
-				problem |= gnome_print_ps2_putc (ps2, ' ');
-				g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-			}
-			problem = gnome_print_ps2_puts (ps2, "0 0] ");
-			g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-			/* xyshow */
-			problem = gnome_print_ps2_puts (ps2, "xyshow" EOL);
-			g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-		} else {
-			problem = gnome_print_ps2_puts (ps2, ")show" EOL);
-			g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
-		}
+		problem = gnome_print_ps2_show (ps2, pgl, start, len);
+		g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
 	}
 
 	if (!identity) {
